# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

from . import _utilities
import typing
# Export this package's modules as members:
from .database import *
from .default_privileg import *
from .default_privileges import *
from .extension import *
from .function import *
from .get_schemas import *
from .get_sequences import *
from .get_tables import *
from .grant import *
from .grant_role import *
from .physical_replication_slot import *
from .provider import *
from .publication import *
from .replication_slot import *
from .role import *
from .schema import *
from .server import *
from .subscription import *
from .user_mapping import *
from ._inputs import *
from . import outputs

# Make subpackages available:
if typing.TYPE_CHECKING:
    import pulumi_postgresql.config as __config
    config = __config
else:
    config = _utilities.lazy_import('pulumi_postgresql.config')

_utilities.register(
    resource_modules="""
[
 {
  "pkg": "postgresql",
  "mod": "index/database",
  "fqn": "pulumi_postgresql",
  "classes": {
   "postgresql:index/database:Database": "Database"
  }
 },
 {
  "pkg": "postgresql",
  "mod": "index/defaultPrivileg",
  "fqn": "pulumi_postgresql",
  "classes": {
   "postgresql:index/defaultPrivileg:DefaultPrivileg": "DefaultPrivileg"
  }
 },
 {
  "pkg": "postgresql",
  "mod": "index/defaultPrivileges",
  "fqn": "pulumi_postgresql",
  "classes": {
   "postgresql:index/defaultPrivileges:DefaultPrivileges": "DefaultPrivileges"
  }
 },
 {
  "pkg": "postgresql",
  "mod": "index/extension",
  "fqn": "pulumi_postgresql",
  "classes": {
   "postgresql:index/extension:Extension": "Extension"
  }
 },
 {
  "pkg": "postgresql",
  "mod": "index/function",
  "fqn": "pulumi_postgresql",
  "classes": {
   "postgresql:index/function:Function": "Function"
  }
 },
 {
  "pkg": "postgresql",
  "mod": "index/grant",
  "fqn": "pulumi_postgresql",
  "classes": {
   "postgresql:index/grant:Grant": "Grant"
  }
 },
 {
  "pkg": "postgresql",
  "mod": "index/grantRole",
  "fqn": "pulumi_postgresql",
  "classes": {
   "postgresql:index/grantRole:GrantRole": "GrantRole"
  }
 },
 {
  "pkg": "postgresql",
  "mod": "index/physicalReplicationSlot",
  "fqn": "pulumi_postgresql",
  "classes": {
   "postgresql:index/physicalReplicationSlot:PhysicalReplicationSlot": "PhysicalReplicationSlot"
  }
 },
 {
  "pkg": "postgresql",
  "mod": "index/publication",
  "fqn": "pulumi_postgresql",
  "classes": {
   "postgresql:index/publication:Publication": "Publication"
  }
 },
 {
  "pkg": "postgresql",
  "mod": "index/replicationSlot",
  "fqn": "pulumi_postgresql",
  "classes": {
   "postgresql:index/replicationSlot:ReplicationSlot": "ReplicationSlot"
  }
 },
 {
  "pkg": "postgresql",
  "mod": "index/role",
  "fqn": "pulumi_postgresql",
  "classes": {
   "postgresql:index/role:Role": "Role"
  }
 },
 {
  "pkg": "postgresql",
  "mod": "index/schema",
  "fqn": "pulumi_postgresql",
  "classes": {
   "postgresql:index/schema:Schema": "Schema"
  }
 },
 {
  "pkg": "postgresql",
  "mod": "index/server",
  "fqn": "pulumi_postgresql",
  "classes": {
   "postgresql:index/server:Server": "Server"
  }
 },
 {
  "pkg": "postgresql",
  "mod": "index/subscription",
  "fqn": "pulumi_postgresql",
  "classes": {
   "postgresql:index/subscription:Subscription": "Subscription"
  }
 },
 {
  "pkg": "postgresql",
  "mod": "index/userMapping",
  "fqn": "pulumi_postgresql",
  "classes": {
   "postgresql:index/userMapping:UserMapping": "UserMapping"
  }
 }
]
""",
    resource_packages="""
[
 {
  "pkg": "postgresql",
  "token": "pulumi:providers:postgresql",
  "fqn": "pulumi_postgresql",
  "class": "Provider"
 }
]
"""
)



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = ['FunctionArgs', 'Function']

@pulumi.input_type
class FunctionArgs:
    def __init__(__self__, *,
                 body: pulumi.Input[str],
                 args: Optional[pulumi.Input[Sequence[pulumi.Input['FunctionArgArgs']]]] = None,
                 database: Optional[pulumi.Input[str]] = None,
                 drop_cascade: Optional[pulumi.Input[bool]] = None,
                 language: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 parallel: Optional[pulumi.Input[str]] = None,
                 returns: Optional[pulumi.Input[str]] = None,
                 schema: Optional[pulumi.Input[str]] = None,
                 security_definer: Optional[pulumi.Input[bool]] = None,
                 strict: Optional[pulumi.Input[bool]] = None,
                 volatility: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a Function resource.
        :param pulumi.Input[str] body: Function body.
               This should be the body content within the `AS $$` and the final `$$`. It will also accept the `AS $$` and `$$` if added.
        :param pulumi.Input[Sequence[pulumi.Input['FunctionArgArgs']]] args: List of arguments for the function.
        :param pulumi.Input[str] database: The database where the function is located.
               If not specified, the function is created in the current database.
        :param pulumi.Input[bool] drop_cascade: True to automatically drop objects that depend on the function (such as
               operators or triggers), and in turn all objects that depend on those objects. Default is false.
        :param pulumi.Input[str] language: The function programming language. Can be one of internal, sql, c, plpgsql. Default is plpgsql.
        :param pulumi.Input[str] name: The name of the function.
        :param pulumi.Input[str] parallel: Indicates if the function is parallel safe. Can be one of UNSAFE, RESTRICTED, or SAFE. Default is UNSAFE.
        :param pulumi.Input[str] returns: Type that the function returns. It can be computed from the OUT arguments. Default is void.
        :param pulumi.Input[str] schema: The schema where the function is located.
               If not specified, the function is created in the current schema.
        :param pulumi.Input[bool] security_definer: If the function should execute with the permissions of the owner, rather than the permissions of the caller. Default is false.
        :param pulumi.Input[bool] strict: If the function should always return NULL when any of the inputs is NULL. Default is false.
        :param pulumi.Input[str] volatility: Defines the volatility of the function. Can be one of VOLATILE, STABLE, or IMMUTABLE. Default is VOLATILE.
        """
        pulumi.set(__self__, "body", body)
        if args is not None:
            pulumi.set(__self__, "args", args)
        if database is not None:
            pulumi.set(__self__, "database", database)
        if drop_cascade is not None:
            pulumi.set(__self__, "drop_cascade", drop_cascade)
        if language is not None:
            pulumi.set(__self__, "language", language)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if parallel is not None:
            pulumi.set(__self__, "parallel", parallel)
        if returns is not None:
            pulumi.set(__self__, "returns", returns)
        if schema is not None:
            pulumi.set(__self__, "schema", schema)
        if security_definer is not None:
            pulumi.set(__self__, "security_definer", security_definer)
        if strict is not None:
            pulumi.set(__self__, "strict", strict)
        if volatility is not None:
            pulumi.set(__self__, "volatility", volatility)

    @property
    @pulumi.getter
    def body(self) -> pulumi.Input[str]:
        """
        Function body.
        This should be the body content within the `AS $$` and the final `$$`. It will also accept the `AS $$` and `$$` if added.
        """
        return pulumi.get(self, "body")

    @body.setter
    def body(self, value: pulumi.Input[str]):
        pulumi.set(self, "body", value)

    @property
    @pulumi.getter
    def args(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['FunctionArgArgs']]]]:
        """
        List of arguments for the function.
        """
        return pulumi.get(self, "args")

    @args.setter
    def args(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['FunctionArgArgs']]]]):
        pulumi.set(self, "args", value)

    @property
    @pulumi.getter
    def database(self) -> Optional[pulumi.Input[str]]:
        """
        The database where the function is located.
        If not specified, the function is created in the current database.
        """
        return pulumi.get(self, "database")

    @database.setter
    def database(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "database", value)

    @property
    @pulumi.getter(name="dropCascade")
    def drop_cascade(self) -> Optional[pulumi.Input[bool]]:
        """
        True to automatically drop objects that depend on the function (such as
        operators or triggers), and in turn all objects that depend on those objects. Default is false.
        """
        return pulumi.get(self, "drop_cascade")

    @drop_cascade.setter
    def drop_cascade(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "drop_cascade", value)

    @property
    @pulumi.getter
    def language(self) -> Optional[pulumi.Input[str]]:
        """
        The function programming language. Can be one of internal, sql, c, plpgsql. Default is plpgsql.
        """
        return pulumi.get(self, "language")

    @language.setter
    def language(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "language", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the function.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def parallel(self) -> Optional[pulumi.Input[str]]:
        """
        Indicates if the function is parallel safe. Can be one of UNSAFE, RESTRICTED, or SAFE. Default is UNSAFE.
        """
        return pulumi.get(self, "parallel")

    @parallel.setter
    def parallel(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "parallel", value)

    @property
    @pulumi.getter
    def returns(self) -> Optional[pulumi.Input[str]]:
        """
        Type that the function returns. It can be computed from the OUT arguments. Default is void.
        """
        return pulumi.get(self, "returns")

    @returns.setter
    def returns(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "returns", value)

    @property
    @pulumi.getter
    def schema(self) -> Optional[pulumi.Input[str]]:
        """
        The schema where the function is located.
        If not specified, the function is created in the current schema.
        """
        return pulumi.get(self, "schema")

    @schema.setter
    def schema(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "schema", value)

    @property
    @pulumi.getter(name="securityDefiner")
    def security_definer(self) -> Optional[pulumi.Input[bool]]:
        """
        If the function should execute with the permissions of the owner, rather than the permissions of the caller. Default is false.
        """
        return pulumi.get(self, "security_definer")

    @security_definer.setter
    def security_definer(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "security_definer", value)

    @property
    @pulumi.getter
    def strict(self) -> Optional[pulumi.Input[bool]]:
        """
        If the function should always return NULL when any of the inputs is NULL. Default is false.
        """
        return pulumi.get(self, "strict")

    @strict.setter
    def strict(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "strict", value)

    @property
    @pulumi.getter
    def volatility(self) -> Optional[pulumi.Input[str]]:
        """
        Defines the volatility of the function. Can be one of VOLATILE, STABLE, or IMMUTABLE. Default is VOLATILE.
        """
        return pulumi.get(self, "volatility")

    @volatility.setter
    def volatility(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "volatility", value)


@pulumi.input_type
class _FunctionState:
    def __init__(__self__, *,
                 args: Optional[pulumi.Input[Sequence[pulumi.Input['FunctionArgArgs']]]] = None,
                 body: Optional[pulumi.Input[str]] = None,
                 database: Optional[pulumi.Input[str]] = None,
                 drop_cascade: Optional[pulumi.Input[bool]] = None,
                 language: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 parallel: Optional[pulumi.Input[str]] = None,
                 returns: Optional[pulumi.Input[str]] = None,
                 schema: Optional[pulumi.Input[str]] = None,
                 security_definer: Optional[pulumi.Input[bool]] = None,
                 strict: Optional[pulumi.Input[bool]] = None,
                 volatility: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering Function resources.
        :param pulumi.Input[Sequence[pulumi.Input['FunctionArgArgs']]] args: List of arguments for the function.
        :param pulumi.Input[str] body: Function body.
               This should be the body content within the `AS $$` and the final `$$`. It will also accept the `AS $$` and `$$` if added.
        :param pulumi.Input[str] database: The database where the function is located.
               If not specified, the function is created in the current database.
        :param pulumi.Input[bool] drop_cascade: True to automatically drop objects that depend on the function (such as
               operators or triggers), and in turn all objects that depend on those objects. Default is false.
        :param pulumi.Input[str] language: The function programming language. Can be one of internal, sql, c, plpgsql. Default is plpgsql.
        :param pulumi.Input[str] name: The name of the function.
        :param pulumi.Input[str] parallel: Indicates if the function is parallel safe. Can be one of UNSAFE, RESTRICTED, or SAFE. Default is UNSAFE.
        :param pulumi.Input[str] returns: Type that the function returns. It can be computed from the OUT arguments. Default is void.
        :param pulumi.Input[str] schema: The schema where the function is located.
               If not specified, the function is created in the current schema.
        :param pulumi.Input[bool] security_definer: If the function should execute with the permissions of the owner, rather than the permissions of the caller. Default is false.
        :param pulumi.Input[bool] strict: If the function should always return NULL when any of the inputs is NULL. Default is false.
        :param pulumi.Input[str] volatility: Defines the volatility of the function. Can be one of VOLATILE, STABLE, or IMMUTABLE. Default is VOLATILE.
        """
        if args is not None:
            pulumi.set(__self__, "args", args)
        if body is not None:
            pulumi.set(__self__, "body", body)
        if database is not None:
            pulumi.set(__self__, "database", database)
        if drop_cascade is not None:
            pulumi.set(__self__, "drop_cascade", drop_cascade)
        if language is not None:
            pulumi.set(__self__, "language", language)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if parallel is not None:
            pulumi.set(__self__, "parallel", parallel)
        if returns is not None:
            pulumi.set(__self__, "returns", returns)
        if schema is not None:
            pulumi.set(__self__, "schema", schema)
        if security_definer is not None:
            pulumi.set(__self__, "security_definer", security_definer)
        if strict is not None:
            pulumi.set(__self__, "strict", strict)
        if volatility is not None:
            pulumi.set(__self__, "volatility", volatility)

    @property
    @pulumi.getter
    def args(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['FunctionArgArgs']]]]:
        """
        List of arguments for the function.
        """
        return pulumi.get(self, "args")

    @args.setter
    def args(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['FunctionArgArgs']]]]):
        pulumi.set(self, "args", value)

    @property
    @pulumi.getter
    def body(self) -> Optional[pulumi.Input[str]]:
        """
        Function body.
        This should be the body content within the `AS $$` and the final `$$`. It will also accept the `AS $$` and `$$` if added.
        """
        return pulumi.get(self, "body")

    @body.setter
    def body(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "body", value)

    @property
    @pulumi.getter
    def database(self) -> Optional[pulumi.Input[str]]:
        """
        The database where the function is located.
        If not specified, the function is created in the current database.
        """
        return pulumi.get(self, "database")

    @database.setter
    def database(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "database", value)

    @property
    @pulumi.getter(name="dropCascade")
    def drop_cascade(self) -> Optional[pulumi.Input[bool]]:
        """
        True to automatically drop objects that depend on the function (such as
        operators or triggers), and in turn all objects that depend on those objects. Default is false.
        """
        return pulumi.get(self, "drop_cascade")

    @drop_cascade.setter
    def drop_cascade(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "drop_cascade", value)

    @property
    @pulumi.getter
    def language(self) -> Optional[pulumi.Input[str]]:
        """
        The function programming language. Can be one of internal, sql, c, plpgsql. Default is plpgsql.
        """
        return pulumi.get(self, "language")

    @language.setter
    def language(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "language", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the function.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def parallel(self) -> Optional[pulumi.Input[str]]:
        """
        Indicates if the function is parallel safe. Can be one of UNSAFE, RESTRICTED, or SAFE. Default is UNSAFE.
        """
        return pulumi.get(self, "parallel")

    @parallel.setter
    def parallel(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "parallel", value)

    @property
    @pulumi.getter
    def returns(self) -> Optional[pulumi.Input[str]]:
        """
        Type that the function returns. It can be computed from the OUT arguments. Default is void.
        """
        return pulumi.get(self, "returns")

    @returns.setter
    def returns(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "returns", value)

    @property
    @pulumi.getter
    def schema(self) -> Optional[pulumi.Input[str]]:
        """
        The schema where the function is located.
        If not specified, the function is created in the current schema.
        """
        return pulumi.get(self, "schema")

    @schema.setter
    def schema(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "schema", value)

    @property
    @pulumi.getter(name="securityDefiner")
    def security_definer(self) -> Optional[pulumi.Input[bool]]:
        """
        If the function should execute with the permissions of the owner, rather than the permissions of the caller. Default is false.
        """
        return pulumi.get(self, "security_definer")

    @security_definer.setter
    def security_definer(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "security_definer", value)

    @property
    @pulumi.getter
    def strict(self) -> Optional[pulumi.Input[bool]]:
        """
        If the function should always return NULL when any of the inputs is NULL. Default is false.
        """
        return pulumi.get(self, "strict")

    @strict.setter
    def strict(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "strict", value)

    @property
    @pulumi.getter
    def volatility(self) -> Optional[pulumi.Input[str]]:
        """
        Defines the volatility of the function. Can be one of VOLATILE, STABLE, or IMMUTABLE. Default is VOLATILE.
        """
        return pulumi.get(self, "volatility")

    @volatility.setter
    def volatility(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "volatility", value)


class Function(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 args: Optional[pulumi.Input[Sequence[pulumi.Input[Union['FunctionArgArgs', 'FunctionArgArgsDict']]]]] = None,
                 body: Optional[pulumi.Input[str]] = None,
                 database: Optional[pulumi.Input[str]] = None,
                 drop_cascade: Optional[pulumi.Input[bool]] = None,
                 language: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 parallel: Optional[pulumi.Input[str]] = None,
                 returns: Optional[pulumi.Input[str]] = None,
                 schema: Optional[pulumi.Input[str]] = None,
                 security_definer: Optional[pulumi.Input[bool]] = None,
                 strict: Optional[pulumi.Input[bool]] = None,
                 volatility: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        The ``Function`` resource creates and manages a function on a PostgreSQL
        server.

        ## Usage

        ```python
        import pulumi
        import pulumi_postgresql as postgresql

        increment = postgresql.Function("increment",
            name="increment",
            args=[{
                "name": "i",
                "type": "integer",
            }],
            returns="integer",
            language="plpgsql",
            body=\"\"\"BEGIN
            RETURN i + 1;
        END;
        \"\"\")
        ```

        ## Import

        It is possible to import a `postgresql_function` resource with the following
        command:

        ```sh
        $ pulumi import postgresql:index/function:Function function_foo "my_database.my_schema.my_function_name(arguments)"
        ```
        Where `my_database` is the name of the database containing the schema,
        `my_schema` is the name of the schema in the PostgreSQL database, `my_function_name` is the function name to be imported, `arguments` is the argument signature of the function including all non OUT types and
        `postgresql_schema.function_foo` is the name of the resource whose state will be
        populated as a result of the command.

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[Union['FunctionArgArgs', 'FunctionArgArgsDict']]]] args: List of arguments for the function.
        :param pulumi.Input[str] body: Function body.
               This should be the body content within the `AS $$` and the final `$$`. It will also accept the `AS $$` and `$$` if added.
        :param pulumi.Input[str] database: The database where the function is located.
               If not specified, the function is created in the current database.
        :param pulumi.Input[bool] drop_cascade: True to automatically drop objects that depend on the function (such as
               operators or triggers), and in turn all objects that depend on those objects. Default is false.
        :param pulumi.Input[str] language: The function programming language. Can be one of internal, sql, c, plpgsql. Default is plpgsql.
        :param pulumi.Input[str] name: The name of the function.
        :param pulumi.Input[str] parallel: Indicates if the function is parallel safe. Can be one of UNSAFE, RESTRICTED, or SAFE. Default is UNSAFE.
        :param pulumi.Input[str] returns: Type that the function returns. It can be computed from the OUT arguments. Default is void.
        :param pulumi.Input[str] schema: The schema where the function is located.
               If not specified, the function is created in the current schema.
        :param pulumi.Input[bool] security_definer: If the function should execute with the permissions of the owner, rather than the permissions of the caller. Default is false.
        :param pulumi.Input[bool] strict: If the function should always return NULL when any of the inputs is NULL. Default is false.
        :param pulumi.Input[str] volatility: Defines the volatility of the function. Can be one of VOLATILE, STABLE, or IMMUTABLE. Default is VOLATILE.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: FunctionArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        The ``Function`` resource creates and manages a function on a PostgreSQL
        server.

        ## Usage

        ```python
        import pulumi
        import pulumi_postgresql as postgresql

        increment = postgresql.Function("increment",
            name="increment",
            args=[{
                "name": "i",
                "type": "integer",
            }],
            returns="integer",
            language="plpgsql",
            body=\"\"\"BEGIN
            RETURN i + 1;
        END;
        \"\"\")
        ```

        ## Import

        It is possible to import a `postgresql_function` resource with the following
        command:

        ```sh
        $ pulumi import postgresql:index/function:Function function_foo "my_database.my_schema.my_function_name(arguments)"
        ```
        Where `my_database` is the name of the database containing the schema,
        `my_schema` is the name of the schema in the PostgreSQL database, `my_function_name` is the function name to be imported, `arguments` is the argument signature of the function including all non OUT types and
        `postgresql_schema.function_foo` is the name of the resource whose state will be
        populated as a result of the command.

        :param str resource_name: The name of the resource.
        :param FunctionArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(FunctionArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 args: Optional[pulumi.Input[Sequence[pulumi.Input[Union['FunctionArgArgs', 'FunctionArgArgsDict']]]]] = None,
                 body: Optional[pulumi.Input[str]] = None,
                 database: Optional[pulumi.Input[str]] = None,
                 drop_cascade: Optional[pulumi.Input[bool]] = None,
                 language: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 parallel: Optional[pulumi.Input[str]] = None,
                 returns: Optional[pulumi.Input[str]] = None,
                 schema: Optional[pulumi.Input[str]] = None,
                 security_definer: Optional[pulumi.Input[bool]] = None,
                 strict: Optional[pulumi.Input[bool]] = None,
                 volatility: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = FunctionArgs.__new__(FunctionArgs)

            __props__.__dict__["args"] = args
            if body is None and not opts.urn:
                raise TypeError("Missing required property 'body'")
            __props__.__dict__["body"] = body
            __props__.__dict__["database"] = database
            __props__.__dict__["drop_cascade"] = drop_cascade
            __props__.__dict__["language"] = language
            __props__.__dict__["name"] = name
            __props__.__dict__["parallel"] = parallel
            __props__.__dict__["returns"] = returns
            __props__.__dict__["schema"] = schema
            __props__.__dict__["security_definer"] = security_definer
            __props__.__dict__["strict"] = strict
            __props__.__dict__["volatility"] = volatility
        super(Function, __self__).__init__(
            'postgresql:index/function:Function',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            args: Optional[pulumi.Input[Sequence[pulumi.Input[Union['FunctionArgArgs', 'FunctionArgArgsDict']]]]] = None,
            body: Optional[pulumi.Input[str]] = None,
            database: Optional[pulumi.Input[str]] = None,
            drop_cascade: Optional[pulumi.Input[bool]] = None,
            language: Optional[pulumi.Input[str]] = None,
            name: Optional[pulumi.Input[str]] = None,
            parallel: Optional[pulumi.Input[str]] = None,
            returns: Optional[pulumi.Input[str]] = None,
            schema: Optional[pulumi.Input[str]] = None,
            security_definer: Optional[pulumi.Input[bool]] = None,
            strict: Optional[pulumi.Input[bool]] = None,
            volatility: Optional[pulumi.Input[str]] = None) -> 'Function':
        """
        Get an existing Function resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[Union['FunctionArgArgs', 'FunctionArgArgsDict']]]] args: List of arguments for the function.
        :param pulumi.Input[str] body: Function body.
               This should be the body content within the `AS $$` and the final `$$`. It will also accept the `AS $$` and `$$` if added.
        :param pulumi.Input[str] database: The database where the function is located.
               If not specified, the function is created in the current database.
        :param pulumi.Input[bool] drop_cascade: True to automatically drop objects that depend on the function (such as
               operators or triggers), and in turn all objects that depend on those objects. Default is false.
        :param pulumi.Input[str] language: The function programming language. Can be one of internal, sql, c, plpgsql. Default is plpgsql.
        :param pulumi.Input[str] name: The name of the function.
        :param pulumi.Input[str] parallel: Indicates if the function is parallel safe. Can be one of UNSAFE, RESTRICTED, or SAFE. Default is UNSAFE.
        :param pulumi.Input[str] returns: Type that the function returns. It can be computed from the OUT arguments. Default is void.
        :param pulumi.Input[str] schema: The schema where the function is located.
               If not specified, the function is created in the current schema.
        :param pulumi.Input[bool] security_definer: If the function should execute with the permissions of the owner, rather than the permissions of the caller. Default is false.
        :param pulumi.Input[bool] strict: If the function should always return NULL when any of the inputs is NULL. Default is false.
        :param pulumi.Input[str] volatility: Defines the volatility of the function. Can be one of VOLATILE, STABLE, or IMMUTABLE. Default is VOLATILE.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _FunctionState.__new__(_FunctionState)

        __props__.__dict__["args"] = args
        __props__.__dict__["body"] = body
        __props__.__dict__["database"] = database
        __props__.__dict__["drop_cascade"] = drop_cascade
        __props__.__dict__["language"] = language
        __props__.__dict__["name"] = name
        __props__.__dict__["parallel"] = parallel
        __props__.__dict__["returns"] = returns
        __props__.__dict__["schema"] = schema
        __props__.__dict__["security_definer"] = security_definer
        __props__.__dict__["strict"] = strict
        __props__.__dict__["volatility"] = volatility
        return Function(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def args(self) -> pulumi.Output[Optional[Sequence['outputs.FunctionArg']]]:
        """
        List of arguments for the function.
        """
        return pulumi.get(self, "args")

    @property
    @pulumi.getter
    def body(self) -> pulumi.Output[str]:
        """
        Function body.
        This should be the body content within the `AS $$` and the final `$$`. It will also accept the `AS $$` and `$$` if added.
        """
        return pulumi.get(self, "body")

    @property
    @pulumi.getter
    def database(self) -> pulumi.Output[str]:
        """
        The database where the function is located.
        If not specified, the function is created in the current database.
        """
        return pulumi.get(self, "database")

    @property
    @pulumi.getter(name="dropCascade")
    def drop_cascade(self) -> pulumi.Output[Optional[bool]]:
        """
        True to automatically drop objects that depend on the function (such as
        operators or triggers), and in turn all objects that depend on those objects. Default is false.
        """
        return pulumi.get(self, "drop_cascade")

    @property
    @pulumi.getter
    def language(self) -> pulumi.Output[Optional[str]]:
        """
        The function programming language. Can be one of internal, sql, c, plpgsql. Default is plpgsql.
        """
        return pulumi.get(self, "language")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[str]:
        """
        The name of the function.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def parallel(self) -> pulumi.Output[Optional[str]]:
        """
        Indicates if the function is parallel safe. Can be one of UNSAFE, RESTRICTED, or SAFE. Default is UNSAFE.
        """
        return pulumi.get(self, "parallel")

    @property
    @pulumi.getter
    def returns(self) -> pulumi.Output[str]:
        """
        Type that the function returns. It can be computed from the OUT arguments. Default is void.
        """
        return pulumi.get(self, "returns")

    @property
    @pulumi.getter
    def schema(self) -> pulumi.Output[str]:
        """
        The schema where the function is located.
        If not specified, the function is created in the current schema.
        """
        return pulumi.get(self, "schema")

    @property
    @pulumi.getter(name="securityDefiner")
    def security_definer(self) -> pulumi.Output[Optional[bool]]:
        """
        If the function should execute with the permissions of the owner, rather than the permissions of the caller. Default is false.
        """
        return pulumi.get(self, "security_definer")

    @property
    @pulumi.getter
    def strict(self) -> pulumi.Output[Optional[bool]]:
        """
        If the function should always return NULL when any of the inputs is NULL. Default is false.
        """
        return pulumi.get(self, "strict")

    @property
    @pulumi.getter
    def volatility(self) -> pulumi.Output[Optional[str]]:
        """
        Defines the volatility of the function. Can be one of VOLATILE, STABLE, or IMMUTABLE. Default is VOLATILE.
        """
        return pulumi.get(self, "volatility")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = ['ServerArgs', 'Server']

@pulumi.input_type
class ServerArgs:
    def __init__(__self__, *,
                 fdw_name: pulumi.Input[str],
                 server_name: pulumi.Input[str],
                 drop_cascade: Optional[pulumi.Input[bool]] = None,
                 options: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 server_owner: Optional[pulumi.Input[str]] = None,
                 server_type: Optional[pulumi.Input[str]] = None,
                 server_version: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a Server resource.
        :param pulumi.Input[str] fdw_name: The name of the foreign-data wrapper that manages the server.
               Changing this value
               will force the creation of a new resource as this value can only be set
               when the foreign server is created.
        :param pulumi.Input[str] server_name: The name of the foreign server to be created.
        :param pulumi.Input[bool] drop_cascade: When true, will drop objects that depend on the server (such as user mappings), and in turn all objects that depend on those objects . (Default: false)
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] options: This clause specifies the options for the server. The options typically define the connection details of the server, but the actual names and values are dependent on the server's foreign-data wrapper.
        :param pulumi.Input[str] server_owner: By default, the user who defines the server becomes its owner. Set this value to configure the new owner of the foreign server.
        :param pulumi.Input[str] server_type: Optional server type, potentially useful to foreign-data wrappers.
               Changing this value
               will force the creation of a new resource as this value can only be set
               when the foreign server is created.
        :param pulumi.Input[str] server_version: Optional server version, potentially useful to foreign-data wrappers.
        """
        pulumi.set(__self__, "fdw_name", fdw_name)
        pulumi.set(__self__, "server_name", server_name)
        if drop_cascade is not None:
            pulumi.set(__self__, "drop_cascade", drop_cascade)
        if options is not None:
            pulumi.set(__self__, "options", options)
        if server_owner is not None:
            pulumi.set(__self__, "server_owner", server_owner)
        if server_type is not None:
            pulumi.set(__self__, "server_type", server_type)
        if server_version is not None:
            pulumi.set(__self__, "server_version", server_version)

    @property
    @pulumi.getter(name="fdwName")
    def fdw_name(self) -> pulumi.Input[str]:
        """
        The name of the foreign-data wrapper that manages the server.
        Changing this value
        will force the creation of a new resource as this value can only be set
        when the foreign server is created.
        """
        return pulumi.get(self, "fdw_name")

    @fdw_name.setter
    def fdw_name(self, value: pulumi.Input[str]):
        pulumi.set(self, "fdw_name", value)

    @property
    @pulumi.getter(name="serverName")
    def server_name(self) -> pulumi.Input[str]:
        """
        The name of the foreign server to be created.
        """
        return pulumi.get(self, "server_name")

    @server_name.setter
    def server_name(self, value: pulumi.Input[str]):
        pulumi.set(self, "server_name", value)

    @property
    @pulumi.getter(name="dropCascade")
    def drop_cascade(self) -> Optional[pulumi.Input[bool]]:
        """
        When true, will drop objects that depend on the server (such as user mappings), and in turn all objects that depend on those objects . (Default: false)
        """
        return pulumi.get(self, "drop_cascade")

    @drop_cascade.setter
    def drop_cascade(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "drop_cascade", value)

    @property
    @pulumi.getter
    def options(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        This clause specifies the options for the server. The options typically define the connection details of the server, but the actual names and values are dependent on the server's foreign-data wrapper.
        """
        return pulumi.get(self, "options")

    @options.setter
    def options(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "options", value)

    @property
    @pulumi.getter(name="serverOwner")
    def server_owner(self) -> Optional[pulumi.Input[str]]:
        """
        By default, the user who defines the server becomes its owner. Set this value to configure the new owner of the foreign server.
        """
        return pulumi.get(self, "server_owner")

    @server_owner.setter
    def server_owner(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "server_owner", value)

    @property
    @pulumi.getter(name="serverType")
    def server_type(self) -> Optional[pulumi.Input[str]]:
        """
        Optional server type, potentially useful to foreign-data wrappers.
        Changing this value
        will force the creation of a new resource as this value can only be set
        when the foreign server is created.
        """
        return pulumi.get(self, "server_type")

    @server_type.setter
    def server_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "server_type", value)

    @property
    @pulumi.getter(name="serverVersion")
    def server_version(self) -> Optional[pulumi.Input[str]]:
        """
        Optional server version, potentially useful to foreign-data wrappers.
        """
        return pulumi.get(self, "server_version")

    @server_version.setter
    def server_version(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "server_version", value)


@pulumi.input_type
class _ServerState:
    def __init__(__self__, *,
                 drop_cascade: Optional[pulumi.Input[bool]] = None,
                 fdw_name: Optional[pulumi.Input[str]] = None,
                 options: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 server_name: Optional[pulumi.Input[str]] = None,
                 server_owner: Optional[pulumi.Input[str]] = None,
                 server_type: Optional[pulumi.Input[str]] = None,
                 server_version: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering Server resources.
        :param pulumi.Input[bool] drop_cascade: When true, will drop objects that depend on the server (such as user mappings), and in turn all objects that depend on those objects . (Default: false)
        :param pulumi.Input[str] fdw_name: The name of the foreign-data wrapper that manages the server.
               Changing this value
               will force the creation of a new resource as this value can only be set
               when the foreign server is created.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] options: This clause specifies the options for the server. The options typically define the connection details of the server, but the actual names and values are dependent on the server's foreign-data wrapper.
        :param pulumi.Input[str] server_name: The name of the foreign server to be created.
        :param pulumi.Input[str] server_owner: By default, the user who defines the server becomes its owner. Set this value to configure the new owner of the foreign server.
        :param pulumi.Input[str] server_type: Optional server type, potentially useful to foreign-data wrappers.
               Changing this value
               will force the creation of a new resource as this value can only be set
               when the foreign server is created.
        :param pulumi.Input[str] server_version: Optional server version, potentially useful to foreign-data wrappers.
        """
        if drop_cascade is not None:
            pulumi.set(__self__, "drop_cascade", drop_cascade)
        if fdw_name is not None:
            pulumi.set(__self__, "fdw_name", fdw_name)
        if options is not None:
            pulumi.set(__self__, "options", options)
        if server_name is not None:
            pulumi.set(__self__, "server_name", server_name)
        if server_owner is not None:
            pulumi.set(__self__, "server_owner", server_owner)
        if server_type is not None:
            pulumi.set(__self__, "server_type", server_type)
        if server_version is not None:
            pulumi.set(__self__, "server_version", server_version)

    @property
    @pulumi.getter(name="dropCascade")
    def drop_cascade(self) -> Optional[pulumi.Input[bool]]:
        """
        When true, will drop objects that depend on the server (such as user mappings), and in turn all objects that depend on those objects . (Default: false)
        """
        return pulumi.get(self, "drop_cascade")

    @drop_cascade.setter
    def drop_cascade(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "drop_cascade", value)

    @property
    @pulumi.getter(name="fdwName")
    def fdw_name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the foreign-data wrapper that manages the server.
        Changing this value
        will force the creation of a new resource as this value can only be set
        when the foreign server is created.
        """
        return pulumi.get(self, "fdw_name")

    @fdw_name.setter
    def fdw_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "fdw_name", value)

    @property
    @pulumi.getter
    def options(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        This clause specifies the options for the server. The options typically define the connection details of the server, but the actual names and values are dependent on the server's foreign-data wrapper.
        """
        return pulumi.get(self, "options")

    @options.setter
    def options(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "options", value)

    @property
    @pulumi.getter(name="serverName")
    def server_name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the foreign server to be created.
        """
        return pulumi.get(self, "server_name")

    @server_name.setter
    def server_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "server_name", value)

    @property
    @pulumi.getter(name="serverOwner")
    def server_owner(self) -> Optional[pulumi.Input[str]]:
        """
        By default, the user who defines the server becomes its owner. Set this value to configure the new owner of the foreign server.
        """
        return pulumi.get(self, "server_owner")

    @server_owner.setter
    def server_owner(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "server_owner", value)

    @property
    @pulumi.getter(name="serverType")
    def server_type(self) -> Optional[pulumi.Input[str]]:
        """
        Optional server type, potentially useful to foreign-data wrappers.
        Changing this value
        will force the creation of a new resource as this value can only be set
        when the foreign server is created.
        """
        return pulumi.get(self, "server_type")

    @server_type.setter
    def server_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "server_type", value)

    @property
    @pulumi.getter(name="serverVersion")
    def server_version(self) -> Optional[pulumi.Input[str]]:
        """
        Optional server version, potentially useful to foreign-data wrappers.
        """
        return pulumi.get(self, "server_version")

    @server_version.setter
    def server_version(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "server_version", value)


class Server(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 drop_cascade: Optional[pulumi.Input[bool]] = None,
                 fdw_name: Optional[pulumi.Input[str]] = None,
                 options: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 server_name: Optional[pulumi.Input[str]] = None,
                 server_owner: Optional[pulumi.Input[str]] = None,
                 server_type: Optional[pulumi.Input[str]] = None,
                 server_version: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        The ``Server`` resource creates and manages a foreign server on a PostgreSQL server.

        ## Usage

        ```python
        import pulumi
        import pulumi_postgresql as postgresql

        ext_postgres_fdw = postgresql.Extension("ext_postgres_fdw", name="postgres_fdw")
        myserver_postgres = postgresql.Server("myserver_postgres",
            server_name="myserver_postgres",
            fdw_name="postgres_fdw",
            options={
                "host": "foo",
                "dbname": "foodb",
                "port": "5432",
            },
            opts = pulumi.ResourceOptions(depends_on=[ext_postgres_fdw]))
        ```

        ```python
        import pulumi
        import pulumi_postgresql as postgresql

        ext_file_fdw = postgresql.Extension("ext_file_fdw", name="file_fdw")
        myserver_file = postgresql.Server("myserver_file",
            server_name="myserver_file",
            fdw_name="file_fdw",
            opts = pulumi.ResourceOptions(depends_on=[ext_file_fdw]))
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] drop_cascade: When true, will drop objects that depend on the server (such as user mappings), and in turn all objects that depend on those objects . (Default: false)
        :param pulumi.Input[str] fdw_name: The name of the foreign-data wrapper that manages the server.
               Changing this value
               will force the creation of a new resource as this value can only be set
               when the foreign server is created.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] options: This clause specifies the options for the server. The options typically define the connection details of the server, but the actual names and values are dependent on the server's foreign-data wrapper.
        :param pulumi.Input[str] server_name: The name of the foreign server to be created.
        :param pulumi.Input[str] server_owner: By default, the user who defines the server becomes its owner. Set this value to configure the new owner of the foreign server.
        :param pulumi.Input[str] server_type: Optional server type, potentially useful to foreign-data wrappers.
               Changing this value
               will force the creation of a new resource as this value can only be set
               when the foreign server is created.
        :param pulumi.Input[str] server_version: Optional server version, potentially useful to foreign-data wrappers.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: ServerArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        The ``Server`` resource creates and manages a foreign server on a PostgreSQL server.

        ## Usage

        ```python
        import pulumi
        import pulumi_postgresql as postgresql

        ext_postgres_fdw = postgresql.Extension("ext_postgres_fdw", name="postgres_fdw")
        myserver_postgres = postgresql.Server("myserver_postgres",
            server_name="myserver_postgres",
            fdw_name="postgres_fdw",
            options={
                "host": "foo",
                "dbname": "foodb",
                "port": "5432",
            },
            opts = pulumi.ResourceOptions(depends_on=[ext_postgres_fdw]))
        ```

        ```python
        import pulumi
        import pulumi_postgresql as postgresql

        ext_file_fdw = postgresql.Extension("ext_file_fdw", name="file_fdw")
        myserver_file = postgresql.Server("myserver_file",
            server_name="myserver_file",
            fdw_name="file_fdw",
            opts = pulumi.ResourceOptions(depends_on=[ext_file_fdw]))
        ```

        :param str resource_name: The name of the resource.
        :param ServerArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ServerArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 drop_cascade: Optional[pulumi.Input[bool]] = None,
                 fdw_name: Optional[pulumi.Input[str]] = None,
                 options: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 server_name: Optional[pulumi.Input[str]] = None,
                 server_owner: Optional[pulumi.Input[str]] = None,
                 server_type: Optional[pulumi.Input[str]] = None,
                 server_version: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ServerArgs.__new__(ServerArgs)

            __props__.__dict__["drop_cascade"] = drop_cascade
            if fdw_name is None and not opts.urn:
                raise TypeError("Missing required property 'fdw_name'")
            __props__.__dict__["fdw_name"] = fdw_name
            __props__.__dict__["options"] = options
            if server_name is None and not opts.urn:
                raise TypeError("Missing required property 'server_name'")
            __props__.__dict__["server_name"] = server_name
            __props__.__dict__["server_owner"] = server_owner
            __props__.__dict__["server_type"] = server_type
            __props__.__dict__["server_version"] = server_version
        super(Server, __self__).__init__(
            'postgresql:index/server:Server',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            drop_cascade: Optional[pulumi.Input[bool]] = None,
            fdw_name: Optional[pulumi.Input[str]] = None,
            options: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
            server_name: Optional[pulumi.Input[str]] = None,
            server_owner: Optional[pulumi.Input[str]] = None,
            server_type: Optional[pulumi.Input[str]] = None,
            server_version: Optional[pulumi.Input[str]] = None) -> 'Server':
        """
        Get an existing Server resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] drop_cascade: When true, will drop objects that depend on the server (such as user mappings), and in turn all objects that depend on those objects . (Default: false)
        :param pulumi.Input[str] fdw_name: The name of the foreign-data wrapper that manages the server.
               Changing this value
               will force the creation of a new resource as this value can only be set
               when the foreign server is created.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] options: This clause specifies the options for the server. The options typically define the connection details of the server, but the actual names and values are dependent on the server's foreign-data wrapper.
        :param pulumi.Input[str] server_name: The name of the foreign server to be created.
        :param pulumi.Input[str] server_owner: By default, the user who defines the server becomes its owner. Set this value to configure the new owner of the foreign server.
        :param pulumi.Input[str] server_type: Optional server type, potentially useful to foreign-data wrappers.
               Changing this value
               will force the creation of a new resource as this value can only be set
               when the foreign server is created.
        :param pulumi.Input[str] server_version: Optional server version, potentially useful to foreign-data wrappers.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ServerState.__new__(_ServerState)

        __props__.__dict__["drop_cascade"] = drop_cascade
        __props__.__dict__["fdw_name"] = fdw_name
        __props__.__dict__["options"] = options
        __props__.__dict__["server_name"] = server_name
        __props__.__dict__["server_owner"] = server_owner
        __props__.__dict__["server_type"] = server_type
        __props__.__dict__["server_version"] = server_version
        return Server(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="dropCascade")
    def drop_cascade(self) -> pulumi.Output[Optional[bool]]:
        """
        When true, will drop objects that depend on the server (such as user mappings), and in turn all objects that depend on those objects . (Default: false)
        """
        return pulumi.get(self, "drop_cascade")

    @property
    @pulumi.getter(name="fdwName")
    def fdw_name(self) -> pulumi.Output[str]:
        """
        The name of the foreign-data wrapper that manages the server.
        Changing this value
        will force the creation of a new resource as this value can only be set
        when the foreign server is created.
        """
        return pulumi.get(self, "fdw_name")

    @property
    @pulumi.getter
    def options(self) -> pulumi.Output[Optional[Mapping[str, str]]]:
        """
        This clause specifies the options for the server. The options typically define the connection details of the server, but the actual names and values are dependent on the server's foreign-data wrapper.
        """
        return pulumi.get(self, "options")

    @property
    @pulumi.getter(name="serverName")
    def server_name(self) -> pulumi.Output[str]:
        """
        The name of the foreign server to be created.
        """
        return pulumi.get(self, "server_name")

    @property
    @pulumi.getter(name="serverOwner")
    def server_owner(self) -> pulumi.Output[str]:
        """
        By default, the user who defines the server becomes its owner. Set this value to configure the new owner of the foreign server.
        """
        return pulumi.get(self, "server_owner")

    @property
    @pulumi.getter(name="serverType")
    def server_type(self) -> pulumi.Output[Optional[str]]:
        """
        Optional server type, potentially useful to foreign-data wrappers.
        Changing this value
        will force the creation of a new resource as this value can only be set
        when the foreign server is created.
        """
        return pulumi.get(self, "server_type")

    @property
    @pulumi.getter(name="serverVersion")
    def server_version(self) -> pulumi.Output[Optional[str]]:
        """
        Optional server version, potentially useful to foreign-data wrappers.
        """
        return pulumi.get(self, "server_version")

